///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright (C) 2010 Brandon M. Smith and Li Zhang                          //
//                                                                           //
// Author:       Brandon M. Smith, Dept. of Computer Sciences, UW-Madison    //
// Email:        bmsmith@cs.wisc.edu                                         //
// Personal web: http://www.cs.wisc.edu/~bmsmith                             //
// Project web:  http://www.cs.wisc.edu/~lizhang/projects/mvstereo/cvpr2009/ //
// Description:                                                              //
//                                                                           //
//   This is a C++ implementation of the multi-view stereo matching          //
// algorithm described in:                                                   //
//                                                                           //
//     Brandon M. Smith, Li Zhang, Hailin Jin. Stereo Matching with Non-     //
//     parametric Smoothness Priors in Feature Space. IEEE Computer Society  //
//     Conference on Computer Vision and Pattern Recognition, June 2009.     //
//                                                                           //
// This program is free software: you can redistribute it and/or modify it   //
// under the terms of the GNU General Public License as published by the     //
// Free Software Foundation, either version 3 of the License, or (at your    //
// option) any later version.                                                //
//                                                                           //
//   This program is distributed in the hope that it will be useful, but     //
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-      //
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public  //
// License for more details.                                                 //
//                                                                           //
//   You should have received a copy of the GNU General Public License along //
// with this program.  If not, see <http://www.gnu.org/licenses/>.           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#include "CmdArgParser.h"


void collectAllArgs(vector <string> &allArgv, int argc, const char *argv[])
{	
	int i = 0; 
	while (i < argc)
	{
		if (strcmp(argv[i], "-af") == 0)
		{
			i++;
			if (i == argc)
			{
				cerr << "collectAllArgs: empty argument filename after -af" << endl;
				exit(0);
			}
			appendCommandArgFromFile(allArgv, argv[i]);
			i++;
		}
		else
		{
			allArgv.push_back(string(argv[i]));
			i++;
		}
	}
}


void appendCommandArgFromFile(vector <string> &allArgv, const char *filename)
{
	ifstream ifs(filename);
	if (ifs.fail())
	{
		cerr << "appendCommandArgFromFile: fail to open " << filename << endl;
		exit(0);
	}

	string s;
	while (true)
	{
		ifs >> s;
		if (ifs.fail())
		{
			break;
		}
		allArgv.push_back(s);		
	}

	ifs.close();
}


bool extractParaFromCommandArg(bool &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			para = true;
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(int &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			sscanf(argv[i+1].c_str(), "%d", &para);
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(unsigned long &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			sscanf(argv[i+1].c_str(), "%u", &para);
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(float &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			sscanf(argv[i+1].c_str(), "%f", &para);				
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(double &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			sscanf(argv[i+1].c_str(), "%lf", &para);				
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(string &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size();
	for (int i = 0; i < argc; i ++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			para = argv[i+1];
			return true;
		}
	}
	return false;
}


bool extractParaFromCommandArg(vector <string> &para, const char flag[], const vector <string> &argv)
{
	int argc = (int)argv.size(); 
	for (int i = 0; i < argc; i++)
	{
		if (strcmp(argv[i].c_str(), flag) == 0)
		{
			i++;
			int N = 0;
			stringstream ss;
			ss << argv[i];
			ss >> N;
			i++;
			if (i+N>argc)
			{
				cerr << "Not enough arguments for " << flag << endl;
				return false;
			}
			para.resize(N);
			for (int j = 0; j<N; j++)
			{				
				para[j] = argv[i+j];
			}
			return true;
		}
	}
	return false;
}			
