///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright (C) 2010 Brandon M. Smith and Li Zhang                          //
//                                                                           //
// Author:       Brandon M. Smith, Dept. of Computer Sciences, UW-Madison    //
// Email:        bmsmith@cs.wisc.edu                                         //
// Personal web: http://www.cs.wisc.edu/~bmsmith                             //
// Project web:  http://www.cs.wisc.edu/~lizhang/projects/mvstereo/cvpr2009/ //
// Description:                                                              //
//                                                                           //
//   This is a C++ implementation of the multi-view stereo matching          //
// algorithm described in:                                                   //
//                                                                           //
//     Brandon M. Smith, Li Zhang, Hailin Jin. Stereo Matching with Non-     //
//     parametric Smoothness Priors in Feature Space. IEEE Computer Society  //
//     Conference on Computer Vision and Pattern Recognition, June 2009.     //
//                                                                           //
// This program is free software: you can redistribute it and/or modify it   //
// under the terms of the GNU General Public License as published by the     //
// Free Software Foundation, either version 3 of the License, or (at your    //
// option) any later version.                                                //
//                                                                           //
//   This program is distributed in the hope that it will be useful, but     //
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-      //
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public  //
// License for more details.                                                 //
//                                                                           //
//   You should have received a copy of the GNU General Public License along //
// with this program.  If not, see <http://www.gnu.org/licenses/>.           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#ifndef EXPANSION_H
#define EXPANSION_H


double computeEnergyAll(
	const Parameters               &params,
	const DataIn                   &dataIn,
	DataOut                        &dataOut);

double computeEnergyPh(
	const Parameters               &params,
	const ImageOf<short>           &imgP, 
	const ImageOf<short>           &imgQ, 
	const ImageOf<short>           &boundP, 
	const ImageOf<short>           &boundQ, 
	const ImageOf<short>           &lblP, 
	const ImageOf<short>           &lblQ,
	const ImageOf<int>             &warpP2Q,
	const ImageOf<float>           &warpP2Qf);

double computeEnergySm(
	const Parameters               &params,
	const ImageOf<short>           &lbl,
	const ImageOf<short>           &img,
	const ImageOf< vector<Coord> > &nbrEdgeSet,
	const ImageOf<float>           &dfArray);

double computeEnergyVs(
	const Parameters               &params,
	const ImageOf<short>           &lblP,
	const ImageOf<short>           &lblQ,
	const ImageOf<int>             &warpP2Q);

int addVariables(
	const ImageOf<short>           &lbl,
	const short                    &alpha,
	Energy                         *energy,
	ImageOf<Energy::Var>           &var);

int addTermsPh(
	const Parameters               &params,
	const ImageOf<short>           &imgP, 
	const ImageOf<short>           &imgQ, 
	const ImageOf<short>           &boundP, 
	const ImageOf<short>           &boundQ, 
	const ImageOf<short>           &lblP, 
	const ImageOf<short>           &lblQ,
	const ImageOf<int>             &warpP2Q,
	const ImageOf<float>           &warpP2Qf,
	const short                    &alpha,
	const ImageOf<Energy::Var>     &varP,
	const ImageOf<Energy::Var>     &varQ,
	Energy                         *energy);

int addTermsSm(
	const Parameters               &params,
	const ImageOf<short>           &img,
	const ImageOf<short>           &segments,
	const ImageOf<short>           &lbl,
	const short                    &alpha,
	const ImageOf<Energy::Var>     &vid,
	const ImageOf< vector<Coord> > &nbrEdgeSet,
	const ImageOf<float>           &dfArray,
	Energy                         *energy);

int addTermsVs(
	const Parameters               &params,
	const ImageOf<short>           &lblP,
	const ImageOf<short>           &lblQ, 
	const ImageOf<int>             &warpP2Q,
	const short                    &alpha,
	const ImageOf<Energy::Var>     &varP,
	const ImageOf<Energy::Var>     &varQ,
	Energy                         *energy);

double expandAlpha(
	const Parameters               &params, 
	const DataIn                   &dataIn, 
	const short                    alpha, 
	const double                   E_old,
	DataOut                        &dataOut);

#endif
