///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright (C) 2010 Brandon M. Smith and Li Zhang                          //
//                                                                           //
// Author:       Brandon M. Smith, Dept. of Computer Sciences, UW-Madison    //
// Email:        bmsmith@cs.wisc.edu                                         //
// Personal web: http://www.cs.wisc.edu/~bmsmith                             //
// Project web:  http://www.cs.wisc.edu/~lizhang/projects/mvstereo/cvpr2009/ //
// Description:                                                              //
//                                                                           //
//   This is a C++ implementation of the multi-view stereo matching          //
// algorithm described in:                                                   //
//                                                                           //
//     Brandon M. Smith, Li Zhang, Hailin Jin. Stereo Matching with Non-     //
//     parametric Smoothness Priors in Feature Space. IEEE Computer Society  //
//     Conference on Computer Vision and Pattern Recognition, June 2009.     //
//                                                                           //
// This program is free software: you can redistribute it and/or modify it   //
// under the terms of the GNU General Public License as published by the     //
// Free Software Foundation, either version 3 of the License, or (at your    //
// option) any later version.                                                //
//                                                                           //
//   This program is distributed in the hope that it will be useful, but     //
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-      //
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public  //
// License for more details.                                                 //
//                                                                           //
//   You should have received a copy of the GNU General Public License along //
// with this program.  If not, see <http://www.gnu.org/licenses/>.           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#ifndef MSTNBR_H
#define MSTNBR_H

#include <float.h>
#include <stdio.h>
#include <valarray>
#include "TVecMat.h"
#include "Image.h"
#include "HeapPQ.h"
#include "data.h"

using std::valarray;


struct PixelKey
{
	int _index;  //pixel index: y*W+x;
	double _key; //min weight from this pixel to the existing spanning tree
};

void computeNbrMST(
	const Parameters         &params, 
	const int                imageIndex, 
	DataIn                   &dataIn);

void mstPrim(
	const Parameters               &params,
	const ImageOf<short>           &img,
	const valarray<Vec3int>        &nbr,
	ImageOf< vector<int> >         &edgeMap);

void readNbrMST(
	const string &filename, 
	ImageOf< vector<Coord> >       &nbrMST);

void writeNbrMST(
	const string                   &filename, 
	const ImageOf< vector<Coord> > &nbrMST);

void swap(
	valarray<PixelKey*>            &pkHeap, 
	PixelKey                       &left, 
	const PixelKey                 &right);

void copy(
	valarray<PixelKey*>            &pkHeap, 
	PixelKey                       &left, 
	const PixelKey                 &right);


////////////////////////////// INLINE FUNCTIONS ///////////////////////////////

inline double square(
	double x)
{
	return x*x;
}

inline double colorDist(
	const short *p, 
	const short *q)
{
	return 0.3*square(p[0]-q[0])+0.59*square(p[1]-q[1])+0.11*square(p[2]-q[2]);
}

inline bool operator < (
	const PixelKey &left, 
	const PixelKey &right) //used in the heap operation
{
	return left._key > right._key;
}

#endif
