///////////////////////////////////////////////////////////////////////////////
//                                                                           //
// Copyright (C) 2010 Brandon M. Smith and Li Zhang                          //
//                                                                           //
// Author:       Brandon M. Smith, Dept. of Computer Sciences, UW-Madison    //
// Email:        bmsmith@cs.wisc.edu                                         //
// Personal web: http://www.cs.wisc.edu/~bmsmith                             //
// Project web:  http://www.cs.wisc.edu/~lizhang/projects/mvstereo/cvpr2009/ //
// Description:                                                              //
//                                                                           //
//   This is a C++ implementation of the multi-view stereo matching          //
// algorithm described in:                                                   //
//                                                                           //
//     Brandon M. Smith, Li Zhang, Hailin Jin. Stereo Matching with Non-     //
//     parametric Smoothness Priors in Feature Space. IEEE Computer Society  //
//     Conference on Computer Vision and Pattern Recognition, June 2009.     //
//                                                                           //
// This program is free software: you can redistribute it and/or modify it   //
// under the terms of the GNU General Public License as published by the     //
// Free Software Foundation, either version 3 of the License, or (at your    //
// option) any later version.                                                //
//                                                                           //
//   This program is distributed in the hope that it will be useful, but     //
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-      //
// ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public  //
// License for more details.                                                 //
//                                                                           //
//   You should have received a copy of the GNU General Public License along //
// with this program.  If not, see <http://www.gnu.org/licenses/>.           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

#ifndef PIXMAP_H
#define PIXMAP_H

#include <stdio.h>

inline void read_comment(FILE *fp)
{
	if (fgetc(fp) == '#')
	{
		while (fgetc(fp) != '\n');
		read_comment(fp);
	}
	else 
	{
		fseek(fp, -1, SEEK_CUR);
	}
}
	
struct PixMapHeader
{
	unsigned int _width, _height, _maxval;
};

void SwapBytes(unsigned char* pixels, int nPixels, int B);

bool LoadPGM(const char *pathname, unsigned char *&pixels, 
			 int &width, int &height, int &maxval);

bool SavePGM(const char *pathname, const unsigned char *pixels, 
			 const int width, const int height, const int maxval);

bool LoadPPM(const char *pathname, unsigned char *&pixels, 
			 int &width, int &height, int &maxval);

bool SavePPM(const char *pathname, const unsigned char *pixels, 
			 const int width, const int height, const int maxval);

bool LoadPFM(const char *pathname, float *&pixels, 
			 int &width, int &height, float &maxval, int &nbands);

bool SavePFM(const char *pathname, const float *pixels, 
			 const int width, const int height, const float maxval, const int nbands);

#endif




